from typing import Any, Callable, Optional, Sequence, Type, TypeVar, Union

from django.http.response import HttpResponseBase

from rest_framework.authentication import BaseAuthentication
from rest_framework.parsers import BaseParser
from rest_framework.permissions import BasePermission
from rest_framework.renderers import BaseRenderer
from rest_framework.schemas.inspectors import ViewInspector
from rest_framework.throttling import BaseThrottle

_VIEW = TypeVar("_VIEW", bound=Callable[..., HttpResponseBase])

def api_view(http_method_names: Optional[Sequence[str]] = ...) -> Callable[[_VIEW], _VIEW]: ...
def renderer_classes(renderer_classes: Sequence[Type[BaseRenderer]]) -> Callable[[_VIEW], _VIEW]: ...
def parser_classes(parser_classes: Sequence[Type[BaseParser]]) -> Callable[[_VIEW], _VIEW]: ...
def authentication_classes(authentication_classes: Sequence[Type[BaseAuthentication]]) -> Callable[[_VIEW], _VIEW]: ...
def throttle_classes(throttle_classes: Sequence[Type[BaseThrottle]]) -> Callable[[_VIEW], _VIEW]: ...
def permission_classes(permission_classes: Sequence[Type[BasePermission]]) -> Callable[[_VIEW], _VIEW]: ...
def schema(view_inspector: Optional[Type[ViewInspector]]) -> Callable[[_VIEW], _VIEW]: ...
def action(
    methods: Optional[Sequence[str]] = ...,
    detail: Optional[bool] = ...,
    url_path: Optional[str] = ...,
    url_name: Optional[str] = ...,
    **kwargs: Any
) -> Callable[[_VIEW], _VIEW]: ...

class MethodMapper(dict):
    def __init__(self, action: Callable, methods: Sequence[str]) -> None: ...
    def _map(self, method: str, func: Callable) -> Callable: ...
    def get(self, func: Callable) -> Callable: ...  # type: ignore
    def post(self, func: Callable) -> Callable: ...
    def put(self, func: Callable) -> Callable: ...
    def patch(self, func: Callable) -> Callable: ...
    def delete(self, func: Callable) -> Callable: ...
    def head(self, func: Callable) -> Callable: ...
    def options(self, func: Callable) -> Callable: ...
    def trace(self, func: Callable) -> Callable: ...

def detail_route(methods: Optional[Sequence[str]] = ..., **kwargs: Any) -> Callable[[_VIEW], _VIEW]: ...
def list_route(methods: Optional[Sequence[str]] = ..., **kwargs: Any) -> Callable[[_VIEW], _VIEW]: ...
